@@ -4,12 +4,17 @@
 
 svyby<-function(formula, by, design, FUN,..., deff=FALSE, keep.var=TRUE,
                 keep.names=TRUE,verbose=FALSE,vartype=c("se","cv","cvpct","var"),
-                drop.empty.groups=TRUE){
+                drop.empty.groups=TRUE, covmat=FALSE){
 
   if (inherits(by, "formula"))
     byfactors<-model.frame(by, design$variables, na.action=na.pass)
   else
     byfactors<-as.data.frame(by)
+
+  if(covmat){
+    if (!inherits(design,"svyrep.design"))
+      stop("covmat=TRUE not implemented for this design type")
+  }
   
   byfactor<-do.call("interaction", byfactors)
   uniques <- which(!duplicated(byfactors))
@@ -33,31 +38,42 @@ svyby<-function(formula, by, design, FUN,..., deff=FALSE, keep.var=TRUE,
   
   if (keep.var){
       unwrap <-function(x){
-          rval<-c(statistics=unclass(x))
-          nvar<-length(rval)
-          rval<-c(rval,c(se=SE(x),
-                         cv=cv(x),
-                         `cv%`=cv(x)*100,
-                         var=SE(x)^2)[rep((nvartype-1)*(nvar),each=nvar)+(1:nvar)])
-          if(!is.null(attr(x,"deff")))
-              rval<-c(rval,DEff=deff(x))
-          rval
+        rval<-c(statistics=if(covmat) unclass(x[[1]]) else unclass(x))
+        nvar<-length(rval)
+        rval<-c(rval,c(se=SE(x),
+                       cv=cv(x),
+                       `cv%`=cv(x)*100,
+                       var=SE(x)^2)[rep((nvartype-1)*(nvar),each=nvar)+(1:nvar)])
+        if(!is.null(attr(x,"deff")))
+          rval<-c(rval,DEff=deff(x))
+        rval
       }
 
-                       
-  
-    rval<-t(sapply(uniques,
-                   function(i) {
-                       if(verbose) print(as.character(byfactor[i]))
-                       if (inherits(formula,"formula"))
-                           data<-formula
-                       else
-                           data<-subset(formula, byfactor %in% byfactor[i])
-                       unwrap(FUN(data,
-                                  design[byfactor %in% byfactor[i],],
-                                  deff=deff,...)) }
-                   ))
-  } else {
+      ## In dire need of refactoring (or rewriting)
+      ## but it seems to work.
+      results<-lapply(uniques,
+                      function(i){
+                        if(verbose) print(as.character(byfactor[i]))
+                        if (inherits(formula,"formula"))
+                          data<-formula
+                        else
+                          data<-subset(formula, byfactor %in% byfactor[i])
+                        if (covmat) {
+                          FUN(data,
+                              design[byfactor %in% byfactor[i],],
+                              deff=deff,...,return.replicates=TRUE)
+                        } else {
+                          FUN(data,
+                              design[byfactor %in% byfactor[i],],
+                              deff=deff,...)
+                        }
+                      })
+      rval<-t(sapply(results, unwrap))
+      if (covmat) {
+        replicates<-do.call(cbind,lapply(results,"[[","replicates"))
+        covmat.mat<-svrVar(replicates,design$scale,design$rscales)
+      }
+    } else {
       unwrap2 <- function(x){
           if(!is.null(attr(x, "deff")))
               c(statistic = unclass(x),
@@ -87,11 +103,29 @@ svyby<-function(formula, by, design, FUN,..., deff=FALSE, keep.var=TRUE,
   else
     rval <-cbind(byfactors[uniques,,drop=FALSE], statistic=rval)
 
+  expand.index<-function(index,reps,x=FALSE){
+    print(index)
+    ns<-max(index)
+    if (x){
+      i<-matrix(1:(ns*reps),ncol=reps)
+      rval<-t(i[index,])
+      
+    } else{
+      i<-matrix(1:(ns*reps), ncol=reps, nrow=ns, byrow=TRUE)
+      rval<- i[index,]
+    }
+    print(rval)
+    as.vector(rval)
+  }
 
   if(drop.empty.groups){
       if (keep.names)
           rownames(rval)<-paste(byfactor[uniques])
       rval<-rval[order(byfactor[uniques]),]
+      if(covmat){
+        i<-expand.index(order(byfactor[uniques]),nstats)
+        covmat.mat<-covmat.mat[i,i]
+      }
   } else {
       a<-do.call("expand.grid", lapply(byfactors,function(f) levels(as.factor(f))))
       a<-cbind(a,matrix(NA, ncol=nr, nrow=nrow(a)))
@@ -100,10 +134,14 @@ svyby<-function(formula, by, design, FUN,..., deff=FALSE, keep.var=TRUE,
       rval<-a
       if (keep.names)
           rownames(rval)<-levels(byfactor)
+      if(covmat){
+        tmp<-matrix(ncol=nrow(a)*nstats,nrow=nrow(a)*nstats)
+        i<-expand.index(match(byfactor[uniques], levels(byfactor)),nstats,TRUE)
+        tmp[i,i]<-covmat.mat
+        covmat.mat<-tmp
+      }
   }
                   
-
-  
   attr(rval,"svyby")<-list(margins=1:NCOL(byfactors),nstats=nstats,
                            vars=if(keep.var) length(vartype) else 0,
                            deffs=deff,
@@ -113,7 +151,9 @@ svyby<-function(formula, by, design, FUN,..., deff=FALSE, keep.var=TRUE,
                            )
   if (!keep.names)
     rownames(rval)<-1:NROW(rval)
-  
+
+  if(covmat)
+    attr(rval,"var")<-covmat.mat
   attr(rval,"call")<-sys.call()
   class(rval)<-c("svyby","data.frame")
   rval
@@ -145,3 +185,16 @@ deff.svyby<-function(object,...){
     if (!aa$deffs) stop("object does not have design effect information")
     object[,max(aa$margins)+aa$nstats*(1+aa$vars)+(1:aa$nstats)]
 }
+
+vcov.svyby<-function(object,...){
+  rval<-attr(object,"var")
+  if(is.null(rval)){
+    warning("Only diagonal elements of vcov() available")
+    se<-SE(object)
+    if(length(se)>1)
+      rval<-diag(se^2)
+    else
+      rval<-as.matrix(se^2)
+  }
+  rval
+}
